# MergeLinks.tcl
#
# For enabling & disabling the link streams into the merge
# Only for handling 1 merge server, 'though in fact merges can be cascaded
# Assumes first 6 links are connected to the 6 links (SHARCs) and are
# controlled from SHARC.tcl
#
set frame MergeLinks
set frametitle "Merge Links"
set icontitle  "Merge"

set MaxTries 20
set Do_Register_Delete 0


set RegisterServer merge


set Asps 0
set Links 36; # Arbitrary - merge can actually handle 64
set Links 66; # Merge can actually handle 64 but they're displayed on blocks of 6

set LinksUsed 0
for {set link $Asps} {$link < $Links} {incr link} {
   set LinkInUse($link) 0
}

set Configdir "$env(MIDASEXPBASE)"
catch {set Configdir "$env(EXPERIMENTBASE)"}

set Mergeregisterlist {}

lappend Mergeregisterlist [list NetVar.Merge.Debug                  merge   generic  "-nMerge_Debug -tint -fd"]

lappend Mergeregisterlist [list NetVar.DACQ.Stats.TotalCount        merge    generic  "-nDACQ_Stats -tuint -ayes -i1 -l1 -fd"]
lappend Mergeregisterlist [list NetVar.DACQ.Stats.BuffersInUse      merge    generic  "-nDACQ_Stats -tuint -ayes -i20 -l1 -fd"]
    lappend Mergeregisterlist [list NetVar.Merge.LinksAvailable     merge   generic  "-nMerge_LinksAvailable -tint -fd"]

    lappend Mergeregisterlist [list NetVar.Merge.EventBuffer        merge   generic  "-nMerge_EventBuffer -tint -fx"]
    lappend Mergeregisterlist [list NetVar.Merge.Output.CurrPointer merge   generic  "-nMerge_EventCurrent -tint -fd"]

for {set link $Asps} {$link < $Links} {incr link} {
    lappend Mergeregisterlist [list NetVar.Merge.Link$link.InUse    merge   generic  "-nMerge_AspsInUse -tint -a1 -i$link"]
}

for {set link $Asps} {$link < $Links} {incr link} {
    lappend Mergeregisterlist [list NetVar.Merge.Link$link.Port     merge   generic  "-nMerge_LinkPort -tint -a1 -i$link"]
}

for {set link 0} {$link < $Links} {incr link} {
    lappend Mergeregisterlist [list NetVar.Merge.Link$link.Alive    merge   generic  "-nMerge_LinkAlive -tint -a1 -i$link"]
}
lappend Mergeregisterlist [list NetVar.Merge.Links.Alive            merge   generic  "-nMerge_LinkAlive -tint -a1 -i0 -l$Links"]
lappend Mergeregisterlist [list NetVar.Merge.Repeat.Alive           merge   generic  "-nMerge_RepeatAlive -tint"]

proc set-experiment dir {global Configdir; set Configdir $dir; restore_configuration}

proc define-register {class name attributes} {
   global EGallow Do_Register_Delete RegisterServer

   set EGallow_SAVE $EGallow
   set EGallow 0x20006         ;#   already exists

   set register /$RegisterServer/$name
   if {$Do_Register_Delete} {EG "register :pattern '$register' !delete"}
   set result [EG "register :name '$register' :class '$class' !new"]
   EG "register :name '$register' :object '$attributes'"

   set EGallow $EGallow_SAVE

   return $result
}

proc XXX_wait_for_completion {args} {
    global RegisterServer MaxTries

    set tries 1
    set TASK ""

    while {$tries < $MaxTries} {

        if {[llength $args] == 0} then {
            set-footer "Waiting for action to complete ($tries)"
        } else {
            if {[llength $args] > 1} then {
                set TASK "[first $args]."
                set-footer "[second $args] ($tries)"
            } else {
                set-footer "[first $args] ($tries)"
            }
        }

        set z [catch {EG "register :name '/$RegisterServer/[set TASK]EXEC.Action' ?value"} m ]

        if {$z != 0} then {
            midas-warning "Unable to confirm state of action\n$m"
            clear-footer
            return 1
        }

        if {$m == 0} then {
            clear-footer
            return 0
        }
        incr tries
        after 500
    }

    midas-warning "Action has not completed\nGiving up waiting"
    clear-footer
    return 1
}

proc make_registers {registerlist} {
    global EGreport RegisterServer

    set item [first $registerlist]
    set RegisterServer [second $item]
    set r [define-register [third $item] [first $item] "[fourth $item]"]
    if {$r == 0x20006} {return}          ;#  registers already exist

    set EGreport_SAVE $EGreport
    set EGreport -1
    set-footer "defining registers"

    foreach item $registerlist {
        set RegisterServer [second $item]
        define-register [third $item] [first $item] "[fourth $item]"
    }

    clear-footer
    set EGreport $EGreport_SAVE
}

proc restore_configuration {args} {
   global Configdir
   global Asps Links
   global LinksUsed LinkInUse
   global Mergeregisterlist
   if {[llength $args] == 0} {make_registers $Mergeregisterlist}

   set conf MergeLinks_current_configuration
   set z [catch {open [file join $Configdir $conf] r} file]
   if {$z != 0} then {set-footer "$file"; return}

   set configuration [split [read $file] \n]
   close $file

   set n 0
   set LinksUsed 0
   for {set link $Asps} {$link < $Links} {incr link} {
       set LinkInUse($link) [lindex $configuration $n]
       if {$LinkInUse($link) == {}} then {set LinkInUse($link) 0}
       if {$LinkInUse($link) != 0} then {incr LinksUsed}
       incr n
   }
   display-Merge
}

proc load_configuration {} {
    global Asps Links
    global EGreport
    global Configdir
    global LinksUsed LinkInUse
    global RegisterServer
    global Mergeregisterlist

   make_registers $Mergeregisterlist

   set EGreport_SAVE $EGreport
   set EGreport -1

   set RegisterServer merge
   set LinksUsed 0
   for {set link $Asps} {$link < $Links} {incr link} {
      write-register NetVar.Merge.Link$link.InUse $LinkInUse($link)
      if {$LinkInUse($link) != 0} then {incr LinksUsed}
   }

   set conf MergeLinks_current_configuration
   set z [catch {open [file join $Configdir $conf] w} file]
   if {$z != 0} then {set-footer "$file"; return}
   for {set link $Asps} {$link < $Links} {incr link} {
      puts $file $LinkInUse($link)
   }
   close $file

   set EGreport $EGreport_SAVE

}

proc click-link {link} {
    global Links
    global LinksUsed LinkInUse








    set LinksAvailable [read-register NetVar.Merge.LinksAvailable]
    set aspsused 0
    set totalused [expr $aspsused + $LinksUsed]
    set was_avail [expr $LinksAvailable - $totalused]

    set ThisLinkInUse [expr ($LinkInUse($link) + 1) & 1]
    if {$link >= $LinksAvailable && $ThisLinkInUse} then {
        return
    } else {
       if {$ThisLinkInUse == 0} then {
          set used [expr $LinksUsed - 1]
       } else {
          set used [expr $LinksUsed + 1]
       }
    }
    set aspsused 0
    set totalused [expr $aspsused + $used]
    set avail [expr $LinksAvailable - $totalused]
    if {$avail < 0 && $was_avail > $avail} then {
       midas-warning "No more Merge Links available"
       clear-footer
       return
    }
    set LinkInUse($link) $ThisLinkInUse
    set LinksUsed $used
    display-Merge
}

proc display-Merge {} {
   global Asps Links
   global LinksUsed LinkInUse

    set LinksAvailable [read-register NetVar.Merge.LinksAvailable]
    set LinksAlive [read-register NetVar.Merge.Links.Alive]



    set aspsused 0
    set totalused [expr $aspsused + $LinksUsed]
    set avail [expr $LinksAvailable - $totalused]
    :modify UNUSED :value $avail

# enabled no data DarkOliveGreen4
# disabled data DarkGoldenRod1






























   for {set link $Asps} {$link < $Links} {incr link} {

      set count [expr $link / 6]
      set l [expr $link % 6]

      if {$link < $LinksAvailable} then {
        if {$LinkInUse($link) == 0} then {
          if {![lindex $LinksAlive $link]} {
            [:path H$count].link$l  configure -background red
          } else {
            [:path H$count].link$l  configure -background DarkGoldenRod1
          }
        } else {
          if {[lindex $LinksAlive $link]} {
            [:path H$count].link$l  configure -background green
          } else {
            [:path H$count].link$l  configure -background DarkOliveGreen4
          }
        }
      } else {
        if {$LinkInUse($link) == 0} then {
            [:path H$count].link$l  configure -background grey75
        } else {
            [:path H$count].link$l  configure -background grey95
        }
      }
   }
   set-footer "Display Updated [date]"
}

proc do_setup {} {
   global Asps Links
   global LinkInUse
   global RegisterServer

   for {set link $Asps} {$link < $Links} {incr link} {
   }
}

proc auto_start {} {start; close-frame}

proc start args {
   global started 
   if {$started == 0} then {make-frame} else {open-frame}
   restore_configuration
   set started 1
   display-Merge
}

proc link-on {link} {
    global Asps Links
    global LinksUsed LinkInUse




    set InUse $LinkInUse($link)

    if {!$InUse} {
        click-link $link
    }
}

proc link-off {link} {
    global Asps Links
    global LinksUsed LinkInUse




    set InUse $LinkInUse($link)
 
    if {$InUse} {
        click-link $link
    }
}

proc links-on b {
   global EGreport
   set tmp $EGreport
   set EGreport -1
   for {set i 0} {$i < 6} {incr i} {
      link-on [expr $b +$i]
   }
   set EGreport $tmp
}

proc links-off b {
   global EGreport
   set tmp $EGreport
   set EGreport -1
   for {set i 0} {$i < 6} {incr i} {
      link-off [expr $b +$i]
   }
   set EGreport $tmp
}

proc click-all {b act} {
   links-$act [expr 6 * $b]
}

proc make-frame {} {
   global env aspnums Links
   set columns {}
   set c 0;for {set i 0} {$i < $Links} {incr i 6} {lappend columns $c; incr c}

  :frame [self frame-name] :label "[self frame-title]" :show-footer T  :resize F

   :panel M :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
     :hskip 125
     :message M0 :label "Merge Links" :bold T

   :panel B :layout B :padx 20 :pady 10 :after FOOTER
     :layout H; :row-gap 2; :col-gap 5

    :row-gap 10; :col-gap 12
    :next-row T
    :button CONFIGURE :command load_configuration
    :text UNUSED :value 0 :w 4 :deaf T
    :button UPDATE :command "restore_configuration 0"
    :button REFRESH :command "display-Merge"

#     :layout H; :row-gap 10; :col-gap 25

#    :button CONFIGURE :command load_configuration
#    :text UNUSED :value 0 :w 4 :deaf T
#    :button UPDATE :command "restore_configuration 0"

   set count 0
   set link 0

   set panel 0

   :panel P$panel :layout H :padx 20 :pady 10 
     :layout H; 

    foreach canvas $columns {

        set w $count

        :canvas H$w  :w 48 :h 152
        [:path H$w] config -background $env(tkframebackground) 

        set y 15
        for {set i 0} {$i < 6 && $link < $Links} {incr i} {
           entry [:path H$w].link$i -relief flat -width 2 -justify left -foreground black
           :_help COLOURS [:path H$w].link$i Entry
           [:path H$w].link$i insert 0 $link
           :draw window :tag H[set w]link$i :value 30 $y -window [:path H$w].link$i 
           bind [:path H$w].link$i <ButtonPress-1>  "click-link $link"

           incr link
           incr y 20
        }

        incr y 5

        set b 0; set x 14; set colour firebrick2; set act off
        entry [:path H$w].all$b -relief raised -width 2 -justify center -fg black -bg $colour -bd 1
        [:path H$w].all$b insert 0 "[string toupper $act 0 0]"
        :draw window :tag H[set w]all$b :value $x $y -window [:path H$w].all$b
        bind [:path H$w].all$b <ButtonPress-1>  "click-all $w $act"
        :_help ALL [:path H$w].all$b Entry

        set b 1; set x 38; set colour LimeGreen; set act on
        entry [:path H$w].all$b -relief raised -width 2 -justify center -fg black -bg $colour -bd 1
        [:path H$w].all$b insert 0 "[string toupper $act 0 0]"
        :draw window :tag H[set w]all$b :value $x $y -window [:path H$w].all$b
        bind [:path H$w].all$b <ButtonPress-1>  "click-all $w $act"
        :_help ALL [:path H$w].all$b Entry

        incr count
    }

    :show T
}

