# This patch adds custom_PostGo (used by the pattern registers)
# otherwise routine should be as in sys.common.tcl
# vmecontrol is also patched
# routine is in pattern_register.defn.tcl
#
# Also adds lyrtech to some standard routines

if {![info exists LYRTECHPresent]} {set LYRTECHPresent 0}

proc click-vme {action args} {
    global VMEPresent VMEEnable

  if {$VMEPresent == 0} then {return}
  if {$VMEEnable == 0} then {:modify VME :value -1; return}

   starttask vmecontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-reset}
      3    {set p DAQ-setup}
      4    {set p DAQ-custom_PostGo}
    }

   set-footer "sending $p to VME control task"
   set-busy
   if {[llength $args] == 0} then {
       set z [catch {send vmecontrol $p} m]
   } else {
       set z [catch {send vmecontrol $p [first $args]} m]
   }
   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}
   redisplay-vme
   update_button_states
}

# Temporary - I hope - the version in common is buggy - I think, must check
proc execute-command {args} {
    global RDOUTserver RDOUThdwre RDOUTserverList

#    first      action
#    second     message
#    third      parameter
#    fourth     signal number
#    fifth      task name

    if {[llength $args] > 4} then {set TASK [fifth $args]} else {set TASK EXEC}

    foreach server $RDOUTserverList {
        set pid [EG "register :name '/$server/NetVar.$TASK.PID' ?value"]
        if {[llength $args] > 2} {EG "register :name '/$server/NetVar.$TASK.Parameter' :value '[third $args]'"}
        EG "register :name '/$server/NetVar.$TASK.Action' :value '[first $args]'"
        if {[llength $args] > 3} {
            set usrsig [fourth $args]
        } else {
            set usrsig 1
        }
        EG "register :name '/$server/SigTask.$TASK.Usr$usrsig' :object '-t $pid'"
        EG "register :name '/$server/SigTask.$TASK.Usr$usrsig' :value '0'"
        after 25
        set RDOUTserver $server
        if {[llength $args] > 1} {wait_for_completion $TASK "[second $args]"} else {wait_for_completion $TASK ""}
    }
}

proc execute-merge-command {args} {
    global RDOUTserver RDOUThdwre RDOUTserverList

#    first      action
#    second     message
#    third      parameter
#    fourth     signal number
#    fifth      task name

    if {[llength $args] > 4} then {set TASK [fifth $args]} else {set TASK EXEC}

    set failedList {}

    foreach server $RDOUTserverList {
        set pid [EG "register :name '/$server/NetVar.$TASK.PID' ?value"]
        if {[llength $args] > 2} {EG "register :name '/$server/NetVar.$TASK.Parameter' :value '[third $args]'"}
        EG "register :name '/$server/NetVar.$TASK.Action' :value '[first $args]'"
        if {[llength $args] > 3} {
            set usrsig [fourth $args]
        } else {
            set usrsig 1
        }
        EG "register :name '/$server/SigTask.$TASK.Usr$usrsig' :object '-t $pid'"
        EG "register :name '/$server/SigTask.$TASK.Usr$usrsig' :value '0'"
        after 25
        set RDOUTserver $server
        if {[llength $args] > 1} then {
           set done [wait_for_merge_completion $TASK "[second $args]"]
        } else {
           set done [wait_for_merge_completion $TASK ""]
        }
        if {$done == 1} then {
           lappend failedList $server
        }
    }
    foreach server $failedList {
        set pid [EG "register :name '/$server/NetVar.$TASK.PID' ?value"]
        if {[llength $args] > 2} {EG "register :name '/$server/NetVar.$TASK.Parameter' :value '[third $args]'"}
        EG "register :name '/$server/NetVar.$TASK.Action' :value '[first $args]'"
        if {[llength $args] > 3} {
            set usrsig [fourth $args]
        } else {
            set usrsig 1
        }
        EG "register :name '/$server/SigTask.$TASK.Usr$usrsig' :object '-t $pid'"
        EG "register :name '/$server/SigTask.$TASK.Usr$usrsig' :value '0'"
        after 25
        set RDOUTserver $server
        if {[llength $args] > 1} then {
           set done [wait_for_completion $TASK "[second $args]"]
        } else {
           set done [wait_for_completion $TASK ""]
        }
    }
}

set MaxMergeTries 10

proc wait_for_merge_completion {args} {
    global RDOUTserver MaxMergeTries EGreport

    set tries 1
    set TASK EXEC

    while {$tries < $MaxMergeTries} {
        if {[llength $args] == 0} then {
            if {$EGreport != -1} then {set-footer "$RDOUTserver: Waiting for action to complete ($tries)"}
        } else {
            if {[llength $args] > 1} then {
                set TASK [first $args]
                if {$EGreport != -1} then {set-footer "$RDOUTserver: [second $args] ($tries)"}
            } else {
                if {$EGreport != -1} then {set-footer "$RDOUTserver: [first $args] ($tries)"}
            }
        }

        set z [catch {EG "register :name '/$RDOUTserver/NetVar.$TASK.Action' ?value"} m ]

        if {$z != 0} then {
            midas-warning "$RDOUTserver: Unable to confirm state of action\n$m"
            clear-footer
            return -1
        }

        if {$m == 0} then {
            if {$EGreport != -1} clear-footer
            return 0
        }
        incr tries
        after 250
    }

    clear-footer
    return 1
}

proc redisplay-txenable {} {
   global  RDOUTEnable txEnable  TSPresent
   if {$TSPresent == 0} then {return}
   if {$RDOUTEnable == 0} then {:modify TXENABLE :value 0 :colours [list red red]; return}
   set txEnable [read-register NetVar.DACQ.Ctxenable]
   :modify TXENABLE :value $txEnable
   click-merge-special 20
   set txState [read-register NetVar.EXEC.Parameter]
   if {$txState == 0} then {
       :modify TXENABLE :colours [list red red]
   } else {
       :modify TXENABLE :colours [list red green]
   }
}

proc redisplay-ebtxenable {} {
   global  RDOUTEnable ebtxEnable EBPresent
   if {$EBPresent == 0} then {return}
   if {$RDOUTEnable == 0} then {:modify EBTXENABLE :value 0 :colours [list red red]; return}
   set ebtxEnable [read-register NetVar.DACQ.Cebtxenable]
   :modify EBTXENABLE :value $ebtxEnable
   click-merge-special 21
   set txState [read-register NetVar.EXEC.Parameter]
   if {$txState == 0} then {
       :modify EBTXENABLE :colours [list red red]
   } else {
       :modify EBTXENABLE :colours [list red green]
   }
}
proc redisplay-xxxenable {} {
  global CAMACEnable VMEEnable VXIEnable CAMACPresent VMEPresent VXIPresent RDOUTEnable
  global LYRTECHPresent LYRTECHEnable

 if {$CAMACPresent == 1} {
    :modify CAMACENABLE :value $CAMACEnable
    redisplay-camac
 }
 if {$VMEPresent == 1} {
    :modify VMEENABLE :value $VMEEnable
    redisplay-vme
 }
 if {$VXIPresent == 1} {
    :modify VXIENABLE :value $VXIEnable
    redisplay-vxi
 }
 if {$LYRTECHPresent == 1} {
    :modify LYRTECHENABLE :value $LYRTECHEnable
    redisplay-lyrtech
 }
   :modify RDOUTENABLE :value $RDOUTEnable
}

proc click-merge-special {a} {

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

   execute-merge-command $a "" 0 2
}

proc da_state {} {
  global VXIEnable VMEEnable TSEnable RDOUTEnable
  global vxistate vmestate tapestate readoutstate

#    if any one of the controlled subsystems is in the GO state then the
#    overall state is GO button

  set state 0

  if {$VXIEnable == 1 && $vxistate == 1}     then {set state 1}
  if {$VMEEnable == 1 && $vmestate == 1}     then {set state 1}
  if {$TSEnable == 1 && $tapestate == 1}     then {set state 1}
  if {$RDOUTEnable == 1 && $readoutstate == 1}     then {set state 1}

  global LYRTECHEnable
  global lyrtechstate
  if {$LYRTECHEnable == 1 && $lyrtechstate == 1}     then {set state 1}

  return $state
}

proc update_button_states {} {
  global VXIEnable VMEEnable TSEnable RDOUTEnable
  global vxistate vmestate tapestate readoutstate

#    if any one of the controlled subsystems is in the GO state then the
#    overall GO button should be disabled

  if {$VXIEnable == 1 && $vxistate == 1}     then {disable_go; return}
  if {$VMEEnable == 1 && $vmestate == 1}     then {disable_go; return}
  if {$TSEnable == 1 && $tapestate == 2}     then {disable_go; return}
  if {$RDOUTEnable == 1 && $readoutstate == 1}     then {disable_go; return}

  global LYRTECHEnable
  global lyrtechstate
  if {$LYRTECHEnable == 1 && $lyrtechstate == 1}     then {disable_go; return}

  :modify DA-GO :colour green :inactive F
}

proc disable_go {} {:modify DA-GO :colour grey85 :inactive T}

